/*
 * pacman.c
 *
 * Created: 5/19/2019 10:50:33 AM
 * Author : Florin
 */ 

#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>

#include <string.h>
#include <stdio.h>
#include <stdbool.h>
#include <inttypes.h>

#include "resources.h"
#include "ili9341.h"
#include "ili9341gfx.h"

#define POINTCOLOUR PINK

// Buttons constants
#define NUM_BTN 6
#define UP_BUTTON 2
#define DOWN_BUTTON 5
#define RIGHT_BUTTON 4
#define LEFT_BUTTON 3
#define OK_BUTTON 1
#define BACK_BUTTON 0
char button_ports[NUM_BTN] = { PA0, PA1, PA2, PA3, PA4, PA5 };
char button_state = -1;
char button_type = -1;

void IO_init()
{
	// Speaker output
	DDRD |= (1 << PD4);
	

	// Buttons input + pull-up resistor
	PORTA = 0;
	for (int i = 0; i < NUM_BTN; ++i) {
		DDRA &= ~(1 << button_ports[i]);
		PORTA |= (1 << button_ports[i]);
	}
}

ISR(TIMER1_OVF_vect)
{
}

ISR(TIMER1_COMPA_vect)
{
}

ISR(PCINT2_vect)
{
}

ISR(PCINT1_vect)
{
}

void timer1_init()
{	
	TCCR1B |= (1 << CS12) | (1 << CS10);
	TIMSK1 |= (1 << OCIE1A) | (1 << TOIE1);
	TCCR1A |= (1 << COM1A1) | (1 << COM1A0);

	TCCR1A |= (1 << COM1B1) | (1 << WGM10) | (1 << WGM11);
	TCCR1B |= (1 << WGM12) | (1 << WGM13);
}

void timer0_init() 
{
	OCR0A = 0xFA;
	TIMSK0 = (1 << OCIE0A);
	TCCR0B = (1 << CS01) | (1 << CS00) | (1 << WGM02);
}


// Number of the note in the tone
int note_num = 0;
// Duration of playing  the note
int dur = 0;

// Update tone function 
int update_tone(const uint16_t frequencies[], const uint16_t durations[], uint8_t num_notes)
{
	dur -= 25;
	if (dur < 25) {
		OCR1A = 15625 / pgm_read_word(&frequencies[note_num]);
		OCR1B = OCR1A / 2;
		TCNT1 = 0;
		dur = pgm_read_word(&durations[note_num]);
		note_num = (note_num + 1) % num_notes;
	}
	return note_num;
}

// Different delays for tones
const double delay_beginning_tone = 110;
const double delay_other_tones = 50;

// Identification of tones by an integer
#define BEGINNING_TONE 0
#define EATFRUIT_TONE 1
#define EATGHOST_TONE 2
#define CHOMP_TONE 3
#define DEATH_TONE 4

// Function that plays a tone
void play_tone(const uint16_t frequencies[], const uint16_t durations[], uint8_t num_notes, uint8_t tone) {
	dur = 0;
	note_num = 0;
	OCR1A = 0xFFFF;
	while(1)
	{
		if (update_tone(frequencies, durations, num_notes) == num_notes - 1) break;
		if (tone == BEGINNING_TONE) {
			_delay_ms(delay_beginning_tone);
		} else {
			_delay_ms(delay_other_tones);
		}
	}
	OCR1A = 0;
	_delay_ms(1000);
}

// Dimensions of a cell on the map 
#define CELL_DIM (uint16_t) 8
#define W_OFFSET (int16_t) 7
#define H_OFFSET (int16_t) 35
#define H_CELLS (int16_t) 28
#define V_CELLS (int16_t) 31

// Draw walls for the map
void draw_walls(uint16_t color) {
	// Draw map corners
	drawBitmapAlpha(W_OFFSET, H_OFFSET, corner_left_up, CELL_DIM, CELL_DIM, color);
	drawBitmapAlpha(W_OFFSET + (H_CELLS - 1) * CELL_DIM, H_OFFSET, corner_right_up, CELL_DIM, CELL_DIM, color);
	drawBitmapAlpha(W_OFFSET + (H_CELLS - 1) * CELL_DIM, H_OFFSET + (V_CELLS - 1) * CELL_DIM, corner_right_down, CELL_DIM, CELL_DIM, color);
	drawBitmapAlpha(W_OFFSET, H_OFFSET + (V_CELLS - 1) * CELL_DIM, corner_left_down, CELL_DIM, CELL_DIM, color);

	// Upper and lower walls
	for (int i = 1; i <= 26; ++i) {
		if (i != 13 && i != 14) {
			drawBitmapAlpha(W_OFFSET + i * CELL_DIM, H_OFFSET, double_wall_up, CELL_DIM, CELL_DIM, color);
		}
		drawBitmapAlpha(W_OFFSET + i * CELL_DIM, H_OFFSET + (V_CELLS - 1) * CELL_DIM, double_wall_down, CELL_DIM, CELL_DIM, color);
	}
	
	// Left and right walls 
	for (int j = 1; j <= 8; ++j) {
		drawBitmapAlpha(W_OFFSET, H_OFFSET + j * CELL_DIM, double_wall_left, CELL_DIM, CELL_DIM, color);
		drawBitmapAlpha(W_OFFSET + 27 * CELL_DIM, H_OFFSET + j * CELL_DIM, double_wall_right, CELL_DIM, CELL_DIM, color);
	}
	for (int j = 20; j <= 29; ++j) {
		if (j != 24 && j != 25) {
			drawBitmapAlpha(W_OFFSET, H_OFFSET + j * CELL_DIM, double_wall_left, CELL_DIM, CELL_DIM, color);
			drawBitmapAlpha(W_OFFSET + 27 * CELL_DIM, H_OFFSET + j * CELL_DIM, double_wall_right, CELL_DIM, CELL_DIM, color);
		}
	}
	
	// Middle height walls
	for (int i = 1; i <= 4; ++i) {
		for (int j = 9; j <= 15; j += 6) {
			drawBitmapAlpha(W_OFFSET + i * CELL_DIM, H_OFFSET + j * CELL_DIM, double_wall_down, CELL_DIM, CELL_DIM, color);
		}
		for (int j = 13; j <= 19; j += 6) {
			drawBitmapAlpha(W_OFFSET + i * CELL_DIM, H_OFFSET + j * CELL_DIM, double_wall_up, CELL_DIM, CELL_DIM, color);
		}
	}
	
	// Middle height walls
	for (int i = 23; i <= 26; ++i) {
		for (int j = 9; j <= 15; j += 6) {
			drawBitmapAlpha(W_OFFSET + i * CELL_DIM, H_OFFSET + j * CELL_DIM, double_wall_down, CELL_DIM, CELL_DIM, color);
		}
		for (int j = 13; j <= 19; j += 6) {
			drawBitmapAlpha(W_OFFSET + i * CELL_DIM, H_OFFSET + j * CELL_DIM, double_wall_up, CELL_DIM, CELL_DIM, color);
		}
	}
	for (int j = 10; j <= 12; j++) {
			drawBitmapAlpha(W_OFFSET + 5 * CELL_DIM, H_OFFSET + j * CELL_DIM, double_wall_left, CELL_DIM, CELL_DIM, color);
			drawBitmapAlpha(W_OFFSET + 22 * CELL_DIM, H_OFFSET + j * CELL_DIM, double_wall_right, CELL_DIM, CELL_DIM, color);
	}
	for (int j = 16; j <= 18; j++) {
		drawBitmapAlpha(W_OFFSET + 5 * CELL_DIM, H_OFFSET + j * CELL_DIM, double_wall_left, CELL_DIM, CELL_DIM, color);
		drawBitmapAlpha(W_OFFSET + 22 * CELL_DIM, H_OFFSET + j * CELL_DIM, double_wall_right, CELL_DIM, CELL_DIM, color);
	}
	for (int i = 0; i <= 27; i += 27) {
		drawBitmapAlpha(W_OFFSET + i * CELL_DIM, H_OFFSET + 13 * CELL_DIM, double_wall_up, CELL_DIM, CELL_DIM, color);
		drawBitmapAlpha(W_OFFSET + i * CELL_DIM, H_OFFSET + 15 * CELL_DIM, double_wall_down, CELL_DIM, CELL_DIM, color);	
	}
	drawBitmapAlpha(W_OFFSET, H_OFFSET + 9 * CELL_DIM, corner_left_down, CELL_DIM, CELL_DIM, color);
	drawBitmapAlpha(W_OFFSET, H_OFFSET + 19 * CELL_DIM, corner_left_up, CELL_DIM, CELL_DIM, color);
	drawBitmapAlpha(W_OFFSET + 27 * CELL_DIM, H_OFFSET + 9 * CELL_DIM, corner_right_down, CELL_DIM, CELL_DIM, color);
	drawBitmapAlpha(W_OFFSET + 27 * CELL_DIM, H_OFFSET + 19 * CELL_DIM, corner_right_up, CELL_DIM, CELL_DIM, color);
	// Center box
	for (int i = 11; i <= 16; ++i) {
		drawBitmapAlpha(W_OFFSET + i * CELL_DIM, H_OFFSET + 16 * CELL_DIM, double_wall_up, CELL_DIM, CELL_DIM, color);
	}
	for (int i = 11; i <= 16; i += 5) {
		drawBitmapAlpha(W_OFFSET + i * CELL_DIM, H_OFFSET + 12 * CELL_DIM, double_wall_down, CELL_DIM, CELL_DIM, color);
	}
	for (int j = 13; j <= 15; ++j) {
		drawBitmapAlpha(W_OFFSET + 10 * CELL_DIM, H_OFFSET + j * CELL_DIM, double_wall_right, CELL_DIM, CELL_DIM, color);
		drawBitmapAlpha(W_OFFSET + 17 * CELL_DIM, H_OFFSET + j * CELL_DIM, double_wall_left, CELL_DIM, CELL_DIM, color);
	}
	drawBitmapAlpha(W_OFFSET + 10 * CELL_DIM, H_OFFSET + 16 * CELL_DIM, double_corner_left_down, CELL_DIM, CELL_DIM, color);
	drawBitmapAlpha(W_OFFSET + 10 * CELL_DIM, H_OFFSET + 12 * CELL_DIM, double_corner_left_up, CELL_DIM, CELL_DIM, color);
	drawBitmapAlpha(W_OFFSET + 17 * CELL_DIM, H_OFFSET + 16 * CELL_DIM, double_corner_right_down, CELL_DIM, CELL_DIM, color);
	drawBitmapAlpha(W_OFFSET + 17 * CELL_DIM, H_OFFSET + 12 * CELL_DIM, double_corner_right_up, CELL_DIM, CELL_DIM, color);
	drawBitmapAlpha(W_OFFSET + 12 * CELL_DIM, H_OFFSET + 12 * CELL_DIM, double_end_left, CELL_DIM, CELL_DIM, color);
	drawBitmapAlpha(W_OFFSET + 15 * CELL_DIM, H_OFFSET + 12 * CELL_DIM, double_end_right, CELL_DIM, CELL_DIM, color);
	
	// Walls corners
	for (int j = 9; j <= 15; j += 6) {
		drawBitmapAlpha(W_OFFSET + 5 * CELL_DIM, H_OFFSET + j * CELL_DIM, simple_right_up_corner, CELL_DIM, CELL_DIM, color);
	}
	for (int j = 13; j <= 19; j += 6) {
		drawBitmapAlpha(W_OFFSET + 5 * CELL_DIM, H_OFFSET + j * CELL_DIM, simple_right_down_corner, CELL_DIM, CELL_DIM, color);
	}
	for (int j = 9; j <= 15; j += 6) {
		drawBitmapAlpha(W_OFFSET + 22 * CELL_DIM, H_OFFSET + j * CELL_DIM, simple_left_up_corner, CELL_DIM, CELL_DIM, color);
	}
	for (int j = 13; j <= 19; j += 6) {
		drawBitmapAlpha(W_OFFSET + 22 * CELL_DIM, H_OFFSET + j * CELL_DIM, simple_left_down_corner, CELL_DIM, CELL_DIM, color);
	}
	
	// Maze
	drawBitmapAlpha(W_OFFSET, H_OFFSET + 24 * CELL_DIM, left_lab_part, 3 * CELL_DIM, 2 * CELL_DIM, color);
	drawBitmapAlpha(W_OFFSET + 25 * CELL_DIM, H_OFFSET + 24 * CELL_DIM, right_lab_part, 3 * CELL_DIM, 2 * CELL_DIM, color);
	
	drawBitmapAlpha(W_OFFSET + 7 * CELL_DIM, H_OFFSET + 21 * CELL_DIM, h_maze, 5 * CELL_DIM, 2 * CELL_DIM, color);
	drawBitmapAlpha(W_OFFSET + 16 * CELL_DIM, H_OFFSET + 21 * CELL_DIM, h_maze, 5 * CELL_DIM, 2 * CELL_DIM, color);
	
	drawBitmapAlpha(W_OFFSET + 13 * CELL_DIM, H_OFFSET + 0 * CELL_DIM, maze_part1, 2 * CELL_DIM, 5 * CELL_DIM, color);
	
	drawBitmapAlpha(W_OFFSET + 7 * CELL_DIM, H_OFFSET + 2 * CELL_DIM, h_maze2, 5 * CELL_DIM, 3 * CELL_DIM, color);
	drawBitmapAlpha(W_OFFSET + 16 * CELL_DIM, H_OFFSET + 2 * CELL_DIM, h_maze2, 5 * CELL_DIM, 3 * CELL_DIM, color);
	
	drawBitmapAlpha(W_OFFSET + 2 * CELL_DIM, H_OFFSET + 2 * CELL_DIM, h_maze3, 4 * CELL_DIM, 3 * CELL_DIM, color);
	drawBitmapAlpha(W_OFFSET + 22 * CELL_DIM, H_OFFSET + 2 * CELL_DIM, h_maze3, 4 * CELL_DIM, 3 * CELL_DIM, color);
	
	drawBitmapAlpha(W_OFFSET + 2 * CELL_DIM, H_OFFSET + 6 * CELL_DIM, h_maze4, 4 * CELL_DIM, 2 * CELL_DIM, color);
	drawBitmapAlpha(W_OFFSET + 22 * CELL_DIM, H_OFFSET + 6 * CELL_DIM, h_maze4, 4 * CELL_DIM, 2 * CELL_DIM, color);
	
	drawBitmapAlpha(W_OFFSET + 7 * CELL_DIM, H_OFFSET + 15 * CELL_DIM, v_maze1, 2 * CELL_DIM, 5 * CELL_DIM, color);
	drawBitmapAlpha(W_OFFSET + 19 * CELL_DIM, H_OFFSET + 15 * CELL_DIM, v_maze1, 2 * CELL_DIM, 5 * CELL_DIM, color);
	
	drawBitmapAlpha(W_OFFSET + 13 * CELL_DIM, H_OFFSET + 6 * CELL_DIM, v_maze_part1, 2 * CELL_DIM, 5 * CELL_DIM, color);
	drawBitmapAlpha(W_OFFSET + 13 * CELL_DIM, H_OFFSET + 18 * CELL_DIM, v_maze_part1, 2 * CELL_DIM, 5 * CELL_DIM, color);
	drawBitmapAlpha(W_OFFSET + 13 * CELL_DIM, H_OFFSET + 24 * CELL_DIM, v_maze_part1, 2 * CELL_DIM, 5 * CELL_DIM, color);
	
	drawBitmapAlpha(W_OFFSET + 7 * CELL_DIM, H_OFFSET + 24 * CELL_DIM, v_maze_part2, 2 * CELL_DIM, 5 * CELL_DIM, color);
	drawBitmapAlpha(W_OFFSET + 19 * CELL_DIM, H_OFFSET + 24 * CELL_DIM, v_maze_part2, 2 * CELL_DIM, 5 * CELL_DIM, color);
	
	drawBitmapAlpha(W_OFFSET + 7 * CELL_DIM, H_OFFSET + 6 * CELL_DIM, v_maze_part3, 2 * CELL_DIM, 3 * CELL_DIM, color);
	drawBitmapAlpha(W_OFFSET + 19 * CELL_DIM, H_OFFSET + 6 * CELL_DIM, v_maze_part3, 2 * CELL_DIM, 3 * CELL_DIM, color);
	drawBitmapAlpha(W_OFFSET + 7 * CELL_DIM, H_OFFSET + 11 * CELL_DIM, v_maze_part4, 2 * CELL_DIM, 3 * CELL_DIM, color);
	drawBitmapAlpha(W_OFFSET + 19 * CELL_DIM, H_OFFSET + 11 * CELL_DIM, v_maze_part4, 2 * CELL_DIM, 3 * CELL_DIM, color);
	
	drawBitmapAlpha(W_OFFSET + 9 * CELL_DIM, H_OFFSET + 27 * CELL_DIM, v_maze_part5, 3 * CELL_DIM, 2 * CELL_DIM, color);
	drawBitmapAlpha(W_OFFSET + 16 * CELL_DIM, H_OFFSET + 27 * CELL_DIM, v_maze_part6, 3 * CELL_DIM, 2 * CELL_DIM, color);
	
	drawBitmapAlpha(W_OFFSET + 10 * CELL_DIM, H_OFFSET + 6 * CELL_DIM, v_maze_part6, 3 * CELL_DIM, 2 * CELL_DIM, color);
	drawBitmapAlpha(W_OFFSET + 15 * CELL_DIM, H_OFFSET + 6 * CELL_DIM, v_maze_part5, 3 * CELL_DIM, 2 * CELL_DIM, color);

	drawBitmapAlpha(W_OFFSET + 10 * CELL_DIM, H_OFFSET + 24 * CELL_DIM, v_maze_part6, 3 * CELL_DIM, 2 * CELL_DIM, color);
	drawBitmapAlpha(W_OFFSET + 15 * CELL_DIM, H_OFFSET + 24 * CELL_DIM, v_maze_part5, 3 * CELL_DIM, 2 * CELL_DIM, color);
	
	drawBitmapAlpha(W_OFFSET + 10 * CELL_DIM, H_OFFSET + 18 * CELL_DIM, v_maze_part6, 3 * CELL_DIM, 2 * CELL_DIM, color);
	drawBitmapAlpha(W_OFFSET + 15 * CELL_DIM, H_OFFSET + 18 * CELL_DIM, v_maze_part5, 3 * CELL_DIM, 2 * CELL_DIM, color);
	
	drawBitmapAlpha(W_OFFSET + 2 * CELL_DIM, H_OFFSET + 27 * CELL_DIM, v_maze_part7, 5 * CELL_DIM, 2 * CELL_DIM, color);
	drawBitmapAlpha(W_OFFSET + 21 * CELL_DIM, H_OFFSET + 27 * CELL_DIM, v_maze_part8, 5 * CELL_DIM, 2 * CELL_DIM, color);

	drawBitmapAlpha(W_OFFSET + 7 * CELL_DIM, H_OFFSET + 9 * CELL_DIM, v_maze_part9, 5 * CELL_DIM, 2 * CELL_DIM, color);
	drawBitmapAlpha(W_OFFSET + 16 * CELL_DIM, H_OFFSET + 9 * CELL_DIM, v_maze_part10, 5 * CELL_DIM, 2 * CELL_DIM, color);
	
	drawBitmapAlpha(W_OFFSET + 2 * CELL_DIM, H_OFFSET + 21 * CELL_DIM, v_maze_part13, 2 * CELL_DIM, 2 * CELL_DIM, color);
	drawBitmapAlpha(W_OFFSET + 24 * CELL_DIM, H_OFFSET + 21 * CELL_DIM, v_maze_part14, 2 * CELL_DIM, 2 * CELL_DIM, color);	
	drawBitmapAlpha(W_OFFSET + 4 * CELL_DIM, H_OFFSET + 21 * CELL_DIM, v_maze_part11, 2 * CELL_DIM, 5 * CELL_DIM, color);
	drawBitmapAlpha(W_OFFSET + 22 * CELL_DIM, H_OFFSET + 21 * CELL_DIM, v_maze_part12, 2 * CELL_DIM, 5 * CELL_DIM, color);
}

// Draw dots for the map
void draw_dots(uint16_t color) {
	for (int i = 1; i <= 26; i++) {
		drawBitmapAlpha(W_OFFSET + i * CELL_DIM, H_OFFSET + 5 * CELL_DIM, dot, CELL_DIM, CELL_DIM, color);
		drawBitmapAlpha(W_OFFSET + i * CELL_DIM, H_OFFSET + 29 * CELL_DIM, dot, CELL_DIM, CELL_DIM, color);
		if (i != 13 && i != 14) {
			drawBitmapAlpha(W_OFFSET + i * CELL_DIM, H_OFFSET + 1 * CELL_DIM, dot, CELL_DIM, CELL_DIM, color);
			drawBitmapAlpha(W_OFFSET + i * CELL_DIM, H_OFFSET + 20 * CELL_DIM, dot, CELL_DIM, CELL_DIM, color);
		}
		if (i != 6 && i != 7 && i != 8 && i != 13  && i != 14 && i != 19 && i != 20 && i != 21) {
			drawBitmapAlpha(W_OFFSET + i * CELL_DIM, H_OFFSET + 8 * CELL_DIM, dot, CELL_DIM, CELL_DIM, color);
			drawBitmapAlpha(W_OFFSET + i * CELL_DIM, H_OFFSET + 26 * CELL_DIM, dot, CELL_DIM, CELL_DIM, color);
		}
	}
	for (int j = 2; j <= 26; j++) {
		if (j != 5 && j != 20) {
			drawBitmapAlpha(W_OFFSET + 6 * CELL_DIM, H_OFFSET + j * CELL_DIM, dot, CELL_DIM, CELL_DIM, color);
			drawBitmapAlpha(W_OFFSET + 21 * CELL_DIM, H_OFFSET + j * CELL_DIM, dot, CELL_DIM, CELL_DIM, color);
		}
	}
	for (int i = 1; i <= 12 ; i += 11) {
		for (int j = 27; j <= 28; j++) {
			drawBitmapAlpha(W_OFFSET + i * CELL_DIM, H_OFFSET + j * CELL_DIM, dot, CELL_DIM, CELL_DIM, color);
		}
	}
	for (int i = 15; i <= 26 ; i += 11) {
		for (int j = 27; j <= 28; j++) {
			drawBitmapAlpha(W_OFFSET + i * CELL_DIM, H_OFFSET + j * CELL_DIM, dot, CELL_DIM, CELL_DIM, color);
		}
	}
	for (int i = 3; i <= 9 ; i += 6) {
		for (int j = 24; j <= 25; j++) {
			drawBitmapAlpha(W_OFFSET + i * CELL_DIM, H_OFFSET + j * CELL_DIM, dot, CELL_DIM, CELL_DIM, color);
		}
	}
	for (int i = 18; i <= 24; i += 6) {
		for (int j = 24; j <= 25; j++) {
			drawBitmapAlpha(W_OFFSET + i * CELL_DIM, H_OFFSET + j * CELL_DIM, dot, CELL_DIM, CELL_DIM, color);
		}
	}
	for (int i = 1; i <= 12; i += 11) {
		for (int j = 21; j <= 22; j++) {
			drawBitmapAlpha(W_OFFSET + i * CELL_DIM, H_OFFSET + j * CELL_DIM, dot, CELL_DIM, CELL_DIM, color);
		}
	}
	
	for (int i = 15; i <= 29; i += 11) {
		for (int j = 21; j <= 22; j++) {
			drawBitmapAlpha(W_OFFSET + i * CELL_DIM, H_OFFSET + j * CELL_DIM, dot, CELL_DIM, CELL_DIM, color);
		}
	}
	for (int i = 2; i <= 25; i++) {
		if (i != 4 && i != 5 && i != 13 && i != 14 && i != 22 && i != 23) {
			drawBitmapAlpha(W_OFFSET + i * CELL_DIM, H_OFFSET + 23 * CELL_DIM, dot, CELL_DIM, CELL_DIM, color);
		}
	}
	for (int j = 2; j <= 4; j += 2) {
		drawBitmapAlpha(W_OFFSET + 1 * CELL_DIM, H_OFFSET + j * CELL_DIM, dot, CELL_DIM, CELL_DIM, color);
		drawBitmapAlpha(W_OFFSET + 12 * CELL_DIM, H_OFFSET + j * CELL_DIM, dot, CELL_DIM, CELL_DIM, color);
		drawBitmapAlpha(W_OFFSET + 15 * CELL_DIM, H_OFFSET + j * CELL_DIM, dot, CELL_DIM, CELL_DIM, color);
		drawBitmapAlpha(W_OFFSET + 26 * CELL_DIM, H_OFFSET + j * CELL_DIM, dot, CELL_DIM, CELL_DIM, color);
	}
	drawBitmapAlpha(W_OFFSET + 12 * CELL_DIM, H_OFFSET + 3 * CELL_DIM, dot, CELL_DIM, CELL_DIM, color);
	drawBitmapAlpha(W_OFFSET + 15 * CELL_DIM, H_OFFSET + 3 * CELL_DIM, dot, CELL_DIM, CELL_DIM, color);
	
	for (int i = 1; i <= 9; i += 8) {
		for (int j = 6; j <= 7; j++) {
			drawBitmapAlpha(W_OFFSET + i * CELL_DIM, H_OFFSET + j * CELL_DIM, dot, CELL_DIM, CELL_DIM, color);
		}
	}
	for (int i = 18; i <= 26; i += 8) {
		for (int j = 6; j <= 7; j++) {
			drawBitmapAlpha(W_OFFSET + i * CELL_DIM, H_OFFSET + j * CELL_DIM, dot, CELL_DIM, CELL_DIM, color);
		}
	}
}

// The maximum number of lives
#define MAX_LIVES 3

// Lives positions
#define LIVES_Y (H_OFFSET + V_CELLS * CELL_DIM + 2)
#define LIFE1_X W_OFFSET
#define LIFE2_X W_OFFSET + PACMAN_DIM + 2
#define LIFE3_X W_OFFSET + 2 * PACMAN_DIM + 4

uint8_t lives = MAX_LIVES;

// Dimension of a pac-man
#define PACMAN_DIM 13
#define PACMAN_INIT_X (W_OFFSET + 13 * CELL_DIM)
#define PACMAN_INIT_Y (H_OFFSET + 23 * CELL_DIM)
// Pac-man position (in order to draw the image) 
int16_t pacman_x = PACMAN_INIT_X - 3;
int16_t pacman_y = PACMAN_INIT_Y - 3;

// The anterior frame position of the pac-man
int16_t old_pacman_x = PACMAN_INIT_X - 3;
int16_t old_pacman_y = PACMAN_INIT_X - 3;

// Pac-man speed
#define PACMAN_SPEED CELL_DIM

// Pac-man directions
#define UP 0
#define DOWN 1
#define LEFT 2
#define RIGHT 3
// Pac-man type FULL
#define FULL 4
uint8_t pacman_cell_x = 13;
uint8_t pacman_cell_y = 23;
uint8_t pacman_dir = LEFT;
uint8_t pacman_ante_dir = LEFT;
uint8_t pacman_dead = 1;
uint8_t big_dot_gfx_state = 0;
uint8_t pacman_gfx_state = 0;

// Ghost size and constants
#define GHOST_SIZE 14
#define RED_GHOST_INIT_X (W_OFFSET + 13 * CELL_DIM + 2)
#define RED_GHOST_INIT_Y (H_OFFSET + 11 * CELL_DIM - 3)
#define CYAN_GHOST_INIT_X (W_OFFSET + 11 * CELL_DIM + 1)
#define CYAN_GHOST_INIT_Y (H_OFFSET + 14 * CELL_DIM - 3)
#define MAGENTA_GHOST_INIT_X (W_OFFSET + 13 * CELL_DIM + 1)
#define MAGENTA_GHOST_INIT_Y (H_OFFSET + 14 * CELL_DIM - 3)
#define BROWN_GHOST_INIT_X (W_OFFSET + 15 * CELL_DIM + 1)
#define BROWN_GHOST_INIT_Y (H_OFFSET + 14 * CELL_DIM - 3)

uint8_t redghost_x = RED_GHOST_INIT_X;
uint8_t redghost_y = RED_GHOST_INIT_Y;
uint8_t cyanghost_x = CYAN_GHOST_INIT_X;
uint8_t cyanghost_y = CYAN_GHOST_INIT_Y;
uint8_t magentaghost_x = MAGENTA_GHOST_INIT_X;
uint8_t magentaghost_y = MAGENTA_GHOST_INIT_Y;
uint8_t brownghost_x = BROWN_GHOST_INIT_X;
uint8_t brownghost_y = BROWN_GHOST_INIT_Y;

// States of the ghosts
#define FRAME1 0
#define FRAME2 1
#define VULN 2
#define DEAD 3

uint8_t red_state = FRAME1;
uint8_t magenta_state = FRAME1;
uint8_t cyan_state = FRAME1;
uint8_t brown_state = FRAME1;

void draw_ghosts(void) {
	if (red_state == FRAME1) {
		drawRGBBitmap(redghost_x, redghost_y, red_ghost1, GHOST_SIZE, GHOST_SIZE);
		red_state = FRAME2;
	} else if (red_state == FRAME2) {
		drawRGBBitmap(redghost_x, redghost_y, red_ghost2, GHOST_SIZE, GHOST_SIZE);
		red_state = FRAME1;
	}

	if (cyan_state == FRAME1) {
		drawRGBBitmap(cyanghost_x, cyanghost_y, cyan_ghost1, GHOST_SIZE, GHOST_SIZE);
		cyan_state = FRAME2;
	} else if (cyan_state == FRAME2) {
		drawRGBBitmap(cyanghost_x, cyanghost_y, cyan_ghost2, GHOST_SIZE, GHOST_SIZE);
		cyan_state = FRAME1;
	}
	
	if (magenta_state == FRAME1) {
		drawRGBBitmap(magentaghost_x, magentaghost_y, mag_ghost1, GHOST_SIZE, GHOST_SIZE);
		magenta_state = FRAME2;
	} else if (magenta_state == FRAME2) {
		drawRGBBitmap(magentaghost_x, magentaghost_y, mag_ghost2, GHOST_SIZE, GHOST_SIZE);
		magenta_state = FRAME1;
	}
	
	if (brown_state == FRAME1) {
		drawRGBBitmap(brownghost_x, brownghost_y, brown_ghost1, GHOST_SIZE, GHOST_SIZE);
		brown_state = FRAME2;
	} else if (brown_state == FRAME2) {
		drawRGBBitmap(brownghost_x, brownghost_y, brown_ghost2, GHOST_SIZE, GHOST_SIZE);
		brown_state = FRAME1;
	}
}



// Function used to draw the game animations
void draw_animations(uint16_t big_dots_color) {	
	// Draw big dots
	for (int i = 1; i <= 26; i += 25) {
		for (int j = 3; j <= 23; j += 20) {
			if (big_dot_gfx_state == 0) {
				drawBitmapAlpha(W_OFFSET + i * CELL_DIM, H_OFFSET + j * CELL_DIM, big_dot, CELL_DIM, CELL_DIM, big_dots_color);
			} else {
				drawBitmapAlpha(W_OFFSET + i * CELL_DIM, H_OFFSET + j * CELL_DIM, big_dot, CELL_DIM, CELL_DIM, BLACK);
			}
		}
	}
	_delay_ms(1000);
	if (pacman_ante_dir != pacman_dir) {
		drawBitmapAlpha(pacman_x, pacman_y, pacman_full, PACMAN_DIM, PACMAN_DIM, BLACK);
		pacman_ante_dir = pacman_dir;
	}
	// Draw pac-man character and ghosts
	switch (pacman_dir) {
		case UP:
			// Draw ghosts
			draw_ghosts();
			_delay_ms(500);
			draw_ghosts();
			drawBitmapAlpha(pacman_x, pacman_y, pacman_eraser_up, PACMAN_DIM, PACMAN_DIM, BLACK);
			drawBitmapAlpha(pacman_x, pacman_y, pacman_up1, PACMAN_DIM, PACMAN_DIM, YELLOW);
			_delay_ms(1000);
			// Draw ghosts
			draw_ghosts();
			_delay_ms(500);
			draw_ghosts();
			drawBitmapAlpha(pacman_x, pacman_y, pacman_up2, PACMAN_DIM, PACMAN_DIM, YELLOW);
			_delay_ms(1000);
			break;
		case DOWN:
			//drawBitmapAlpha(old_pacman_x, old_pacman_y, pacman_full, PACMAN_DIM, PACMAN_DIM, BLACK);
			//_delay_ms(500);
			// Draw ghost
			draw_ghosts();
			_delay_ms(500);
			draw_ghosts();
			drawBitmapAlpha(pacman_x, pacman_y, pacman_eraser_down, PACMAN_DIM, PACMAN_DIM, BLACK);
			drawBitmapAlpha(pacman_x, pacman_y, pacman_down1, PACMAN_DIM, PACMAN_DIM, YELLOW);
			_delay_ms(1000);
			// Draw ghost
			draw_ghosts();
			_delay_ms(500);
			draw_ghosts();
			drawBitmapAlpha(pacman_x, pacman_y, pacman_down2, PACMAN_DIM, PACMAN_DIM, YELLOW);
			_delay_ms(1000);
			break;
		case LEFT:
			// Draw ghost
			draw_ghosts();
			_delay_ms(500);
			draw_ghosts();
			drawBitmapAlpha(pacman_x, pacman_y, pacman_eraser_left, PACMAN_DIM, PACMAN_DIM, BLACK);
			drawBitmapAlpha(pacman_x, pacman_y, pacman_left1, PACMAN_DIM, PACMAN_DIM, YELLOW);
			_delay_ms(1000);
			// Draw ghost
			draw_ghosts();
			_delay_ms(500);
			draw_ghosts();
			drawBitmapAlpha(pacman_x, pacman_y, pacman_left2, PACMAN_DIM, PACMAN_DIM, YELLOW);
			_delay_ms(1000);
			break;
		case RIGHT:
			// Draw ghost
			draw_ghosts();
			_delay_ms(500);
			draw_ghosts();
			drawBitmapAlpha(pacman_x, pacman_y, pacman_eraser_right, PACMAN_DIM, PACMAN_DIM, BLACK);
			drawBitmapAlpha(pacman_x, pacman_y, pacman_right1, PACMAN_DIM, PACMAN_DIM, YELLOW);
			_delay_ms(1000);
			// Draw ghost
			draw_ghosts();
			_delay_ms(500);
			draw_ghosts();
			drawBitmapAlpha(pacman_x, pacman_y, pacman_right2, PACMAN_DIM, PACMAN_DIM, YELLOW);
			_delay_ms(1000);
			break;
		default:
			// Draw ghost
			draw_ghosts();
			_delay_ms(500);
			draw_ghosts();
			drawBitmapAlpha(pacman_x, pacman_y, pacman_eraser_left, PACMAN_DIM, PACMAN_DIM, BLACK);
			drawBitmapAlpha(pacman_x, pacman_y, pacman_left1, PACMAN_DIM, PACMAN_DIM, YELLOW);
			_delay_ms(1000);
			// Draw ghost
			draw_ghosts();
			_delay_ms(500);
			draw_ghosts();
			drawBitmapAlpha(pacman_x, pacman_y, pacman_left2, PACMAN_DIM, PACMAN_DIM, YELLOW);
			_delay_ms(1000);
	}
}

void life_lost(void) {
		lives--;
		// Draw game over on the screen
		if (lives == 0) {
			lives = MAX_LIVES;
		}
		drawBitmapAlpha(pacman_x, pacman_y, pacman_full, PACMAN_DIM, PACMAN_DIM, YELLOW);
		draw_walls(BLACK);
		draw_walls(WHITE);
		_delay_ms(25);
		draw_walls(BLACK);
		drawBitmapAlpha(pacman_x, pacman_y, pacman_full, PACMAN_DIM, PACMAN_DIM, BLACK);
		draw_walls(BLUE);
}

void draw_lives(void) {
	if (lives >= 1) { 
		drawBitmapAlpha(LIFE1_X, LIVES_Y, pacman_left1, PACMAN_DIM, PACMAN_DIM, YELLOW);
	}
	if (lives >= 2) {
		drawBitmapAlpha(LIFE2_X, LIVES_Y, pacman_left1, PACMAN_DIM, PACMAN_DIM, YELLOW);
	} 
	if (lives >= 3) {
		drawBitmapAlpha(LIFE3_X, LIVES_Y, pacman_left1, PACMAN_DIM, PACMAN_DIM, YELLOW);
	}
}

void update_lives(void) {
	if (lives == 2) {
		drawBitmapAlpha(LIFE3_X, LIVES_Y, pacman_left1, PACMAN_DIM, PACMAN_DIM, BLACK);
		return;
	}
	if (lives == 1) {
		drawBitmapAlpha(LIFE2_X, LIVES_Y, pacman_left1, PACMAN_DIM, PACMAN_DIM, BLACK);
		return;
	}
	if (lives == 0) {
		drawBitmapAlpha(LIFE1_X, LIVES_Y, pacman_left1, PACMAN_DIM, PACMAN_DIM, BLACK);
		return;
	}
}

/*
 * Change the position of the pac-man only at the release of the button
 */
void process_input(void) {
	if ((PINA & (1 << button_ports[DOWN_BUTTON])) == 0) {
		button_state = 0;
		button_type = DOWN_BUTTON;
	} else if ((PINA & (1 << button_ports[UP_BUTTON])) == 0) {
		button_state = 0;
		button_type = UP_BUTTON;
	} else if ((PINA & (1 << button_ports[LEFT_BUTTON])) == 0) {
		button_state = 0;
		button_type = LEFT_BUTTON;
	} else if ((PINA & (1 << button_ports[RIGHT_BUTTON])) == 0) {
		button_state = 0;
		button_type = RIGHT_BUTTON;
	} else if ((PINA & (1 << button_ports[OK_BUTTON])) == 0) {
		button_state = 0;
		button_type = OK_BUTTON;
	} else if ((PINA & (1 << button_ports[BACK_BUTTON])) == 0) {
		button_state = 0;
		button_type = BACK_BUTTON;
	} else if (!button_state) {
		button_state = 1;
	} else {
		button_state = -1;
		button_type = -1;
	}

	if (button_state == 1) {
		switch (button_type) {
			case DOWN_BUTTON:
				pacman_ante_dir = pacman_dir;
				pacman_dir = DOWN;
				break;
			case UP_BUTTON:
				pacman_ante_dir = pacman_dir;
				pacman_dir = UP;
				break;
			case LEFT_BUTTON:
				pacman_ante_dir = pacman_dir;
				pacman_dir = LEFT;
				break;
			case RIGHT_BUTTON:
				pacman_ante_dir = pacman_dir;
				pacman_dir = RIGHT;
				break;
			case OK_BUTTON:
				break;
			case BACK_BUTTON:
				break;
		}
	}
}

/*
 * Timer 0 interrupt for button checking.
 */
ISR(TIMER0_COMPA_vect) {
	process_input();
}

void test_buttons(void)
{
	// Setez pinul PD7 ca pin de iesire.
	DDRD |= (1 << PD7);
	
	PORTA = 0;
	for (int i = 0; i < NUM_BTN; ++i) {
		DDRA &= ~(1 << button_ports[i]);
		PORTA |= (1 << button_ports[i]);
	}
	
	// Sting LED
	PORTD &= ~(1 << PD7);
	
	while (1) {
		if ((PINA & (1 << PA4)) == 0) {
			PORTD |= (1 << PD7);
		}
		PORTD &= ~(1 << PD7);
	}
}

int main(void)
{
	//Initialize I/O
	IO_init();
	// Activate interrupts
	sei();
	
	// Initial driver setup to drive ili9341
	ili9341_init();
	
	// Fill screen with black color
	ili9341_clear(BLACK);
	
	// Create map
	draw_dots(ROSE);
	draw_walls(BLUE);
	draw_lives();
	
	// Initialize Timer 1
	timer1_init();
	timer0_init();
	
	drawBitmapAlpha(pacman_x, pacman_y, pacman_full, PACMAN_DIM, PACMAN_DIM, YELLOW);
	// Play a beginning tone
	play_tone(pacman_beginning, beginning_dur, num_notes_b, BEGINNING_TONE);
	//_delay_ms(100);
	//play_tone(pacman_chomp, chomp_dur, num_notes_c, CHOMP_TONE);
	drawBitmapAlpha(pacman_x, pacman_y, pacman_full, PACMAN_DIM, PACMAN_DIM, BLACK);

	// Draw objects that need to be animated
	while(1) {
		if (pacman_dead == 1) {
			life_lost();
			update_lives();
			pacman_dead = 0;
		}
		
		big_dot_gfx_state = (big_dot_gfx_state + 1) % 2;
		
		old_pacman_x = pacman_x;
		old_pacman_y = pacman_y;
		if (pacman_cell_x == H_CELLS - 2 && pacman_cell_y == V_CELLS - 2 && pacman_dir != RIGHT && pacman_dir != DOWN) {
			goto keep_going;
		}
		if (pacman_cell_x == 1 && pacman_cell_y == V_CELLS - 2 && pacman_dir != LEFT && pacman_dir != DOWN) {
			goto keep_going;
		}
		if (pacman_cell_x == 1 && pacman_cell_y == 1 && pacman_dir != LEFT && pacman_dir != UP) {
			goto keep_going;
		}
		if (pacman_cell_x == H_CELLS - 2 && pacman_cell_y == 1 && pacman_dir != RIGHT && pacman_dir != UP) {
			goto keep_going;
		}
	
		if (pacman_cell_x == 1 && pacman_dir != LEFT) {
			goto keep_going;
		}
		if (pacman_cell_x == H_CELLS - 2 && pacman_dir != RIGHT) {
			goto keep_going;
		}
		if (pacman_cell_y == 1 && pacman_dir != UP) {
			goto keep_going;
		}
		if (pacman_cell_y == V_CELLS - 2 && pacman_dir != DOWN) {
			goto keep_going;
		}
		if (pacman_cell_x > 1 && pacman_cell_x < H_CELLS - 2 && pacman_cell_y > 1 && pacman_cell_y < V_CELLS - 2) {
keep_going:
			if (pacman_dir == UP) {
				pacman_y -= PACMAN_SPEED;
				pacman_cell_y--;
			} else if (pacman_dir == DOWN) {
				pacman_y += PACMAN_SPEED;
				pacman_cell_y++;
			} else if (pacman_dir == LEFT) {
				pacman_x -= PACMAN_SPEED;
				pacman_cell_x--;
			} else if (pacman_dir == RIGHT) {
				pacman_x += PACMAN_SPEED;
				pacman_cell_x++;
			}
			drawBitmapAlpha(old_pacman_x, old_pacman_y, pacman_full, PACMAN_DIM, PACMAN_DIM, BLACK);
		}
		draw_animations(ROSE);
	}
}